/* Gnome Calendar Main Window Object
 * Copyright (C) 2002, 2003  Sebastian Rittau <srittau@jroger.in-berlin.de>
 *
 * $Id: gnomecal-main-window.c,v 1.6 2003/01/11 02:48:08 srittau Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <time.h>

#include <glib.h>

#include <gconf/gconf.h>
#include <gconf/gconf-client.h>

#include <gnome.h>

#include <mimedir/mimedir-vevent.h>

#include "calendar-editor.h"
#include "calendar-widget.h"
#include "gnome-cal.h"
#include "gnomecal-config.h"
#include "gnomecal-goto.h"
#include "gnomecal-main-window.h"
#include "gnomecal-marshal.h"
#include "gnomecal-prefs.h"
#include "gnomecal-ui.h"
#include "timeutil.h"


enum {
	SIGNAL_QUIT,
	SIGNAL_OPEN_NEW,
	SIGNAL_LAST
};


static guint signals[SIGNAL_LAST] = { 0 };


static void gncal_main_window_init		(GncalMainWindow	*window);
static void gncal_main_window_finalize		(GObject		*window);
static void gncal_main_window_class_init	(GncalMainWindowClass	*klass);
static void gncal_main_window_setup		(GncalMainWindow	*window);


struct _GncalMainWindowPriv {
	GtkWidget *calendar;
};


static GnomeAppClass *parent_class = NULL;

/*
 * Class and Object Management
 */

GType
gncal_main_window_get_type (void)
{
	static GType gncal_main_window_type = 0;

	if (!gncal_main_window_type) {
		static const GTypeInfo gncal_main_window_info = {
			sizeof (GncalMainWindowClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) gncal_main_window_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (GncalMainWindow),
			1,    /* n_preallocs */
			(GInstanceInitFunc) gncal_main_window_init,
		};

		gncal_main_window_type = g_type_register_static (GNOME_TYPE_APP,
								 "GncalMainWindow",
								 &gncal_main_window_info,
								 0);
	}

	return gncal_main_window_type;
}

static void
gncal_main_window_class_init (GncalMainWindowClass *klass)
{
	GObjectClass *gobject_class;

	g_return_if_fail (klass != NULL);
	g_return_if_fail (GNCAL_IS_MAIN_WINDOW_CLASS (klass));

	gobject_class = G_OBJECT_CLASS (klass);

	gobject_class->finalize = gncal_main_window_finalize;

	parent_class = g_type_class_peek_parent (klass);

	/* Signal setup */

	signals[SIGNAL_QUIT] =
		g_signal_new ("quit",
			      G_TYPE_FROM_CLASS (klass),
			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
			      G_STRUCT_OFFSET (GncalMainWindowClass, quit),
			      NULL, NULL,
			      gnomecal_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
	signals[SIGNAL_OPEN_NEW] =
		g_signal_new ("open-new",
			      G_TYPE_FROM_CLASS (klass),
			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
			      G_STRUCT_OFFSET (GncalMainWindowClass, open_new),
			      NULL, NULL,
			      gnomecal_marshal_VOID__STRING,
			      G_TYPE_NONE, 1, G_TYPE_STRING);
}

static void
gncal_main_window_init (GncalMainWindow *window)
{
	g_return_if_fail (window != NULL);
	g_return_if_fail (GNCAL_IS_MAIN_WINDOW (window));

	window->priv = g_new0 (GncalMainWindowPriv, 1);

	gncal_main_window_setup (window);
}

static void
gncal_main_window_finalize (GObject *object)
{
	GncalMainWindow *window;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GNCAL_IS_MAIN_WINDOW (object));

	window = GNCAL_MAIN_WINDOW (object);

	g_free (window->priv);

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

/*
 * Internal Methods
 */

static void
gncal_main_window_about (GncalMainWindow *window)
{
	static GtkWidget *about = NULL;

	if (!about) {
		static const gchar *authors[] = {
			"Miguel de Icaza <miguel@kernel.org>",
			"Federico Mena <federico@gimp.org>",
			"Arturo Espinosa <arturo@nuclecu.unam.mx>",
			"Russell Steinthal <rms39@columbia.edu>",
			"Sebastian Rittau <srittau@jroger.in-berlin.de>",
			NULL
		};
		
		const gchar *documenters[] = {
			NULL
		};

		const gchar *translator_credits = _("translator_credits");

		about = gnome_about_new (_("GNOME Calendar"), VERSION,
					 "(C) 1998 Free Software Foundation\n(C) 1998 Red Hat Software, Inc.\n(C) 2002 Sebastian Rittau",
					 _("The GNOME personal calendar and schedule manager"),
					 authors,
					 documenters,
					 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
					 NULL);

		g_signal_connect (G_OBJECT (about), "destroy",
				  G_CALLBACK (gtk_widget_destroyed), &about);
	}

	gtk_window_present (GTK_WINDOW (about));
}

static void
gncal_main_window_set_title (GncalMainWindow *window)
{
	gtk_window_set_title (GTK_WINDOW (window), _("GNOME Calendar")); /* FIXME: add calendar name */
}

/*
 * Menu and Toolbar Callbacks
 */

static void
gncal_main_window_load_file_response (GncalMainWindow *window,
				      gint response_id,
				      GtkFileSelection *dialog)
{
	switch (response_id) {
	case GTK_RESPONSE_OK: {
		GError *error = NULL;
		const gchar *file;

		file = gtk_file_selection_get_filename (dialog);
		if (!gncal_main_window_set_calendar (window, file, &error))
			gncal_ui_show_error (GTK_WINDOW (window), error);
		gtk_widget_destroy (GTK_WIDGET (dialog));

		break;
	}
	case GTK_RESPONSE_CANCEL:
		gtk_widget_destroy (GTK_WIDGET (dialog));
		break;
	case GTK_RESPONSE_DELETE_EVENT:
		break;
	default:
		g_return_if_reached ();
	}
}

static void
gncal_main_window_save_file_response (GncalMainWindow *window,
				      gint response_id,
				      GtkFileSelection *dialog)
{
	switch (response_id) {
	case GTK_RESPONSE_OK: {
		GError *error = NULL;
		const gchar *file;

		file = gtk_file_selection_get_filename (dialog);
		calendar_widget_calendar_save (CALENDAR_WIDGET (window->priv->calendar), file, &error);
		gtk_widget_destroy (GTK_WIDGET (dialog));

		if (error) {
			gncal_ui_show_error (GTK_WINDOW (window), error);
			g_error_free (error);
		}

		break;
	}
	case GTK_RESPONSE_CANCEL:
		gtk_widget_destroy (GTK_WIDGET (dialog));
		break;
	case GTK_RESPONSE_DELETE_EVENT:
		break;
	default:
		g_return_if_reached ();
	}
}

static void
gncal_main_window_new_calendar_cb (GtkMenuItem *item, gpointer data)
{
	g_signal_emit (G_OBJECT (data), signals[SIGNAL_OPEN_NEW], 0, NULL);
}

static void
gncal_main_window_open_calendar_cb (GtkMenuItem *item, gpointer data)
{
	GtkWidget *dialog;

	dialog = gtk_file_selection_new (_("Load calendar"));

	g_signal_connect_swapped (G_OBJECT (dialog), "response",
				  G_CALLBACK (gncal_main_window_load_file_response),
				  data);
	gtk_widget_show (dialog);
}

static void
gncal_main_window_save_calendar_cb (GtkMenuItem *item, gpointer data)
{
	GError *error = NULL;
	GncalMainWindowPriv *priv;

	priv = GNCAL_MAIN_WINDOW (data)->priv;

	calendar_widget_calendar_save (CALENDAR_WIDGET (priv->calendar), NULL, &error);

	if (error) {
		gncal_ui_show_error (NULL, error);
		g_error_free (error);
	}
}

static void
gncal_main_window_save_as_calendar_cb (GtkMenuItem *item, gpointer data)
{
	GtkWidget *dialog;

	dialog = gtk_file_selection_new (_("Save calendar"));

	g_signal_connect_swapped (G_OBJECT (dialog), "response",
				  G_CALLBACK (gncal_main_window_save_file_response),
				  data);
	gtk_widget_show (dialog);
}

static void
gncal_main_window_close_cb (GtkMenuItem *item, gpointer data)
{
	gtk_widget_destroy (GTK_WIDGET (data));
}

static void
gncal_main_window_quit_cb (GtkMenuItem *item, gpointer data)
{
	g_signal_emit (G_OBJECT (data), signals[SIGNAL_QUIT], 0);
}

static void
gncal_main_window_new_appointment_cb (GtkMenuItem *item, GncalMainWindow *window)
{
	GConfClient *gconf;
	GtkWidget *ce;
	MIMEDirVEvent *event;
	MIMEDirDateTime *dtstart_dt, *dtend_dt;
	gint day_begin;
	time_t t, dtstart, dtend;

	/* Load prefs */

	gconf = gconf_client_get_default ();
	day_begin = gconf_client_get_int (gconf, CONFIG_CAL_DAY_START, NULL);
	g_object_unref (G_OBJECT (gconf));

	event = mimedir_vevent_new ();

	/* Default to the day the user is looking at */

	g_object_get (G_OBJECT (event),
		      "dtstart", &dtstart_dt,
		      NULL);
	dtstart = mimedir_datetime_get_time_t (dtstart_dt);
	g_object_unref (G_OBJECT (dtstart_dt));

	t = calendar_widget_get_time (CALENDAR_WIDGET (window->priv->calendar));
	dtstart = time_add_minutes (t, day_begin * 60);
	dtend   = time_add_minutes (dtstart, day_begin * 60 + 30);

	dtstart_dt = mimedir_datetime_new_from_time_t (dtstart);
	dtend_dt   = mimedir_datetime_new_from_time_t (dtend);
	g_object_set (G_OBJECT (event),
		      "dtstart", dtstart_dt,
		      "dtend",   dtend_dt,
		      NULL);
	g_object_unref (G_OBJECT (dtstart_dt));
	g_object_unref (G_OBJECT (dtend_dt));

	ce = calendar_editor_new (event);
	gtk_widget_show (ce);

	/* FIXME: add event to event list */

	g_object_unref (G_OBJECT (event));
}

static void
gncal_main_window_new_appointment_today_cb (GtkMenuItem *item, gpointer data)
{
	MIMEDirVEvent *event;
	GtkWidget *ce;

	event = mimedir_vevent_new ();

	ce = calendar_editor_new (event);
	gtk_widget_show (ce);

	/* FIXME: add event to event list */

	g_object_unref (G_OBJECT (event));
}

static void
gncal_main_window_prefs_cb (GtkMenuItem *item, gpointer data)
{
	static GtkWidget *dialog = NULL;

	if (!dialog) {
		dialog = gncal_prefs_new (GTK_WINDOW (data));
		g_signal_connect (G_OBJECT (dialog), "destroy",
				  G_CALLBACK (gtk_widget_destroyed), &dialog);
	}

	gtk_window_present (GTK_WINDOW (dialog));
}

static void
gncal_main_window_about_cb (GtkMenuItem *item, gpointer data)
{
	gncal_main_window_about (GNCAL_MAIN_WINDOW (data));
}

static void
gncal_main_window_prev_cb (GtkMenuItem *item, gpointer data)
{
	GncalMainWindowPriv *priv;

	priv = GNCAL_MAIN_WINDOW (data)->priv;

	calendar_widget_previous (CALENDAR_WIDGET (priv->calendar));
}

static void
gncal_main_window_today_cb (GtkMenuItem *item, gpointer data)
{
	GncalMainWindowPriv *priv;

	priv = GNCAL_MAIN_WINDOW (data)->priv;

	calendar_widget_today (CALENDAR_WIDGET (priv->calendar));
}
static void
gncal_main_window_next_cb (GtkMenuItem *item, gpointer data)
{
	GncalMainWindowPriv *priv;

	priv = GNCAL_MAIN_WINDOW (data)->priv;

	calendar_widget_next (CALENDAR_WIDGET (priv->calendar));
}

static void
gncal_main_window_goto_changed (GncalGoto *dialog, GncalMainWindow *window)
{
	struct tm tm;

	gncal_goto_get_date (dialog, &tm);

	calendar_widget_goto (CALENDAR_WIDGET (window->priv->calendar), &tm);
}

static void
gncal_main_window_goto_cb (GtkMenuItem *item, gpointer data)
{
	static GtkWidget *dialog = NULL;

	if (!dialog) {
		dialog = gncal_goto_new (GTK_WINDOW (data));

		g_signal_connect (G_OBJECT (dialog), "changed",
				  G_CALLBACK (gncal_main_window_goto_changed), data);
		g_signal_connect (G_OBJECT (dialog), "destroy",
				  G_CALLBACK (gtk_widget_destroyed), &dialog);
	}

	gtk_window_present (GTK_WINDOW (dialog));
}

static void
gncal_main_window_destroyed_cb (GncalMainWindow *window, gpointer data)
{
	GError *error = NULL;

	gncal_main_window_save_calendar (window, &error);

	if (error)
		gncal_ui_show_error (GTK_WINDOW (window), error);
}

/*
 * Setup
 */

static GnomeUIInfo
gncal_main_window_file_menu [] = {
	GNOMEUIINFO_MENU_NEW_ITEM	(N_("_New calendar"), N_("Create a new calendar"), gncal_main_window_new_calendar_cb, NULL),
	GNOMEUIINFO_MENU_OPEN_ITEM	(gncal_main_window_open_calendar_cb,	NULL),
	GNOMEUIINFO_MENU_SAVE_ITEM	(gncal_main_window_save_calendar_cb,	NULL),
	GNOMEUIINFO_MENU_SAVE_AS_ITEM	(gncal_main_window_save_as_calendar_cb,	NULL),

	GNOMEUIINFO_SEPARATOR,

	GNOMEUIINFO_MENU_CLOSE_ITEM	(gncal_main_window_close_cb,	NULL),
	GNOMEUIINFO_MENU_QUIT_ITEM	(gncal_main_window_quit_cb,	NULL),

	GNOMEUIINFO_END
};

static GnomeUIInfo
gncal_main_window_edit_menu [] = {
	{ GNOME_APP_UI_ITEM, N_("_New appointment..."), N_("Create a new appointment"), gncal_main_window_new_appointment_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_NEW, 0, 0, NULL },
	{ GNOME_APP_UI_ITEM, N_("New appointment for _today..."), N_("Create a new appointment for today"), gncal_main_window_new_appointment_today_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_NEW, 0, 0, NULL },

	GNOMEUIINFO_END
};

static GnomeUIInfo
gncal_main_window_settings_menu [] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM (gncal_main_window_prefs_cb, NULL),

	GNOMEUIINFO_END
};

static GnomeUIInfo
gncal_main_window_help_menu [] = {
	GNOMEUIINFO_HELP ("gnomecal"),

	GNOMEUIINFO_MENU_ABOUT_ITEM (gncal_main_window_about_cb, NULL),

	GNOMEUIINFO_END
};

static GnomeUIInfo
gncal_main_window_menu [] = {
	GNOMEUIINFO_MENU_FILE_TREE	(gncal_main_window_file_menu),
	GNOMEUIINFO_MENU_EDIT_TREE	(gncal_main_window_edit_menu),
	GNOMEUIINFO_MENU_SETTINGS_TREE	(gncal_main_window_settings_menu),
	GNOMEUIINFO_MENU_HELP_TREE	(gncal_main_window_help_menu),

	GNOMEUIINFO_END
};

static GnomeUIInfo
gncal_main_window_toolbar [] = {
	GNOMEUIINFO_ITEM_STOCK (N_("New"), N_("Create a new appointment"), gncal_main_window_new_appointment_cb, GTK_STOCK_NEW),

	GNOMEUIINFO_SEPARATOR,

	GNOMEUIINFO_ITEM_STOCK (N_("Prev"), N_("Go back in time"), gncal_main_window_prev_cb, GTK_STOCK_GO_BACK),
	GNOMEUIINFO_ITEM_STOCK (N_("Today"), N_("Go to present time"), gncal_main_window_today_cb, GTK_STOCK_HOME),
	GNOMEUIINFO_ITEM_STOCK (N_("Next"), N_("Go forward in time"), gncal_main_window_next_cb, GTK_STOCK_GO_FORWARD),

	GNOMEUIINFO_SEPARATOR,

	GNOMEUIINFO_ITEM_STOCK (N_("Go to"), N_("Go to a specific date"), gncal_main_window_goto_cb, GTK_STOCK_JUMP_TO),

	GNOMEUIINFO_END
};

static void
gncal_main_window_setup (GncalMainWindow *window)
{
	GncalMainWindowPriv *priv;

	GtkWidget *appbar;

	priv = window->priv;

	g_signal_connect (G_OBJECT (window), "destroy",
			  G_CALLBACK (gncal_main_window_destroyed_cb), NULL);

	gncal_main_window_set_title (window);

	/* Menus */

	gnome_app_create_menus_with_data (GNOME_APP (window), gncal_main_window_menu, window);

	/* Toolbar */

	gnome_app_create_toolbar_with_data (GNOME_APP (window), gncal_main_window_toolbar, window);

	/* Status bar */

        appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
        gnome_app_set_statusbar (GNOME_APP (window), appbar);
	gnome_app_install_menu_hints (GNOME_APP (window), gncal_main_window_menu);

	/* Calendar */

	priv->calendar = calendar_widget_new ();
	gtk_container_set_border_width (GTK_CONTAINER (priv->calendar), 12);
	gnome_app_set_contents (GNOME_APP (window), priv->calendar);
	gtk_widget_show (priv->calendar);
}

/*
 * External Methods
 */

GtkWidget *
gncal_main_window_new (void)
{
	GncalMainWindow *window;

	window = g_object_new (GNCAL_TYPE_MAIN_WINDOW, NULL);

	return GTK_WIDGET (window);
}

gboolean
gncal_main_window_set_calendar (GncalMainWindow *win,
				const gchar *filename,
				GError **error)
{
	g_return_val_if_fail (win != NULL, FALSE);
	g_return_val_if_fail (GNCAL_IS_MAIN_WINDOW (win), FALSE);
	g_return_val_if_fail (filename != NULL, FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	return calendar_widget_calendar_load (CALENDAR_WIDGET (win->priv->calendar), filename, error);
}

gboolean
gncal_main_window_set_calendar_default (GncalMainWindow *win, GError **error)
{
	g_return_val_if_fail (win != NULL, FALSE);
	g_return_val_if_fail (GNCAL_IS_MAIN_WINDOW (win), FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	return calendar_widget_calendar_load_default (CALENDAR_WIDGET (win->priv->calendar), error);
}

void
gncal_main_window_set_calendar_new (GncalMainWindow *win)
{
	g_return_if_fail (win != NULL);
	g_return_if_fail (GNCAL_IS_MAIN_WINDOW (win));

	calendar_widget_calendar_new (CALENDAR_WIDGET (win->priv->calendar));
}

gboolean
gncal_main_window_save_calendar (GncalMainWindow *win, GError **error)
{
	g_return_val_if_fail (win != NULL, FALSE);
	g_return_val_if_fail (GNCAL_IS_MAIN_WINDOW (win), FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	return calendar_widget_calendar_save (CALENDAR_WIDGET (win->priv->calendar), NULL, error);
}
